/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * Copyright 2010-2011 Red Hat, Inc.
 * Copyright 2010-2011 Peng Huang <shawn.p.huang@gmail.com>
 * Copyright 2010-2011 Takao Fujiwara <tfujiwar@redhat.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 2.1 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const IBus = imports.gi.IBus;
const Lang = imports.lang;
const PopupMenu = imports.ui.popupMenu;
const Lightbox = imports.ui.lightbox;
const Main = imports.ui.main;
const Util = imports.misc.util;
const IBUS_PREFIX = imports.misc.config.IBUS_PREFIX;
const IBUS_PKGDATADIR = imports.misc.config.IBUS_PKGDATADIR;

const CandidatePanel = imports.ui.status.ibus.candidatePanel;
const LanguageBar = imports.ui.status.ibus.languageBar;
const PanelBase = imports.ui.status.ibus.panelBase;

const PopupImageMenuItemMarkup = imports.ui.status.ibus.popupMenu.PopupImageMenuItemMarkup;
const Common = imports.ui.status.ibus.common;
const _ = imports.ui.status.ibus.common._;

const ICON_KEYBOARD = 'ibus-keyboard';
const ICON_ENGINE = 'ibus-engine';
const LIGHTBOX_FADE_TIME = 0.1


function Panel(bus, indicator) {
    this._init(bus, indicator);
}

Panel.prototype = {
    __proto__ : PanelBase.PanelBase.prototype,

    _init: function(bus, indicator) {
        this._indicator = indicator;
        this._setup_pid = -1;
        let prefix = IBUS_PREFIX;
        let data_dir = IBUS_PKGDATADIR;
        // this._icons_dir = data_dir + '/icons';
        this._setup_cmd = prefix + '/bin/ibus-setup';
        this._is_restart = false;
        this._active_engine = null;

        if (!this._init_bus(bus)) {
            return;
        }
        // this._bus.config_add_watch('panel')

        this._language_bar = new LanguageBar.LanguageBar(indicator);
        this._language_bar.connect('property-activate',
                                   Lang.bind(this, this._on_language_bar_property_activate));
        this._candidate_panel = new CandidatePanel.CandidatePanel();
        this._candidate_panel.connect('cursor-up',
                                      Lang.bind(this, function(widget) {
                                          this.cursor_up();}));
        this._candidate_panel.connect('cursor-down',
                                      Lang.bind(this, function(widget) {
                                          this.cursor_down();}));
        this._candidate_panel.connect('page-up',
                                      Lang.bind(this, function(widget) {
                                          this.page_up();}));
        this._candidate_panel.connect('page-down',
                                      Lang.bind(this, function(widget) {
                                          this.page_down();}));
        this._candidate_panel.connect('candidate-clicked',
                                      Lang.bind(this, function(widget, index, button, state) {
                                          this.candidate_clicked(index, button, state);}));

        this._indicator.setIcon(ICON_KEYBOARD);
        this._indicator.actor.connect('button-press-event',
                                      Lang.bind(this, this._on_shell_panel_button_press_event));
    },

    _init_bus: function(bus) {
        PanelBase.PanelBase.prototype._init.call(this, bus);
        this._bus = bus;
        this._config = this._bus.get_config();
        this._focus_ic = null;

        // connect bus signal
        if (this._config == null) {
            log('Could not get ibus-gconf.');
            return false;
        }
        this._config.connect('value-changed', this._config_value_changed_cb)
        //this._config.connect('reloaded', this._config_reloaded_cb)
        this._bus.get_connection().signal_subscribe('org.freedesktop.DBus',
                                                    'org.freedesktop.DBus',
                                                    'NameOwnerChanged',
                                                    '/org/freedesktop/DBus',
                                                    IBus.SERVICE_PANEL,
                                                    Gio.DBusSignalFlags.NONE,
                                                    this._config_reloaded_cb,
                                                    null,
                                                    null);
        return true;
    },

    _config_value_changed_cb: function(bus, section, name, value) {
        if (section != 'panel') {
            return;
        }
        if (name == 'lookup_table_orientation') {
            return;
        }
    },

    _config_reloaded_cb: function(bus) {
    },

    _name_owner_changed_cb: function(bus, name, oldname, newname) {
        this._config_reloaded_cb(bus);
    },

    _create_shell_menu_for_im: function() {
        if (this._focus_ic == null) {
            let item = new PopupMenu.PopupImageMenuItem(_("No input window"),
                                                        'dialog-information');
            this._indicator.menu.addMenuItem(item);
            return true;
        } else {
            this._language_bar.create_im_menu_shell();
            return this._create_im_menu_shell();
        }
    },

    _create_shell_menu_for_popup: function() {
        let item = new PopupMenu.PopupImageMenuItem(_("Preferences"),
                                                    'preferences-desktop');
        item.connect('activate',
                     Lang.bind(this, this._preferences_item_shell_activate_cb));
        this._indicator.menu.addMenuItem(item);
        item = new PopupMenu.PopupImageMenuItem(_("Restart"), 'reload');
        item.connect('activate',
                     Lang.bind(this, this._restart_item_shell_activate_cb));
        this._indicator.menu.addMenuItem(item);
        item = new PopupMenu.PopupImageMenuItem(_("Quit"), 'exit');
        item.connect('activate',
                     Lang.bind(this, this._quit_item_shell_activate_cb));
        this._indicator.menu.addMenuItem(item);
    },

    _on_language_bar_property_activate: function(widget, prop_name, prop_state) {
        this.property_activate(prop_name, prop_state);
    },

    _on_shell_panel_button_press_event: function(actor, event) {
        this._indicator.menu.removeAll();
        if (this._create_shell_menu_for_im()) {
            this._indicator.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        }
        this._create_shell_menu_for_popup();
    },

    _check_engines_have_duplicated_lang: function(engines) {
        for (let i = 0; i < engines.length - 1; i++) {
            let engine_i = engines[i];
            if (engine_i == null) {
                continue;
            }
            engine_i.has_duplicated_lang = false;
            let lang_i = IBus.get_language_name(engine_i.language);
            for (let j = i + 1; j < engines.length; j++) {
                let engine_j = engines[j];
                if (engine_j == null) {
                    continue;
                }
                engine_j.has_duplicated_lang = false;
                let lang_j = IBus.get_language_name(engine_j.language);
                if (lang_i == lang_j) {
                    engine_i.has_duplicated_lang = true;
                    engine_j.has_duplicated_lang = true;
                }
            }
        }
        return engines;
    },

    _add_im_off_menu_item: function() {
        let item = new PopupMenu.PopupImageMenuItem(_("Turn off input method"),
                                                    'window-close');
        item._engine = null;
        item.connect('activate',
                     Lang.bind(this, this._im_menu_item_shell_activate_cb));
        if (this._focus_ic == null || !this._focus_ic.is_enabled()) {
            Common.actor_set_sensitive(item.actor, false, item.label);
        } else {
            Common.actor_set_sensitive(item.actor, true, item.label);
        }
        this._indicator.menu.addMenuItem(item);
    },

    _create_im_menu_shell: function() {
        let engines = this._bus.list_active_engines();
        let current_engine = null;
        let item = null;
        current_engine = (this._focus_ic != null && this._focus_ic.get_engine());
        if (current_engine == null) {
            current_engine = (engines && engines[0]);
        }
        engines = this._check_engines_have_duplicated_lang(engines);
        for (let i = 0; i < engines.length; i++) {
            let engine = engines[i];
            if (engine == null) {
                continue;
            }
            let lang = IBus.get_language_name(engine.language);
            if (lang == null) {
                lang = _("Other");
            }
            let text = engine.language.substring(0,2) + '  ' + lang;
            if (engine.has_duplicated_lang) {
                text = text + ' (' + engine.longname + ')';
            }
            let icon = ICON_ENGINE;
            if (engine.icon != null) {
                icon = engine.icon;
            }
            if (current_engine != null && current_engine.name == engine.name) {
                item = new PopupImageMenuItemMarkup('<b>' + text + '</b>', icon);
                item.setShowDot(true);
            } else {
                item = new PopupMenu.PopupImageMenuItem(text, icon);
                item.setShowDot(false);
            }
            item._engine = engine;
            item.connect('activate',
                         Lang.bind(this, this._im_menu_item_shell_activate_cb));
            this._indicator.menu.addMenuItem(item);
        }
        if (engines.length == 0 || engines[0] == null) {
            return false;
        }
        this._add_im_off_menu_item();
        return true;
    },

    _im_menu_item_status_activate_cb: function(item) {
        /* this._focus_ic is null on gnome-shell because focus-in event is 
         * happened. So I moved set_engine in focus_in. */
        if (this._focus_ic == null) {
            if (item._engine != null) {
                this._active_engine = item._engine;
            }
            return;
        }
        if (item._engine != null) {
            this._focus_ic.set_engine(item._engine.name);
        } else {
            this._focus_ic.disable();
        }
    },

    _im_menu_item_shell_activate_cb: function(item, event) {
        this._im_menu_item_status_activate_cb(item);
    },

    _child_watch_cb: function(pid, status, data) {
        if (this._setup_pid == pid) {
            this._setup_pid = -1;
        }
    },

    _preferences_item_shell_activate_cb: function(item, event, user_data) {
        if (this._setup_pid != -1) {
            try {
                Util.trySpawnCommandLine('kill -10 ' + this._setup_pid.toString());
                return;

            } catch (e) {
                this._setup_pid = -1;
            }
        }
        let pid = GLib.spawn_async(null,
                                   [this._setup_cmd, 'ibus-setup'],
                                   null,
                                   GLib.SpawnFlags.DO_NOT_REAP_CHILD, null,
                                   null)[1];
        this._setup_pid = pid;
        GLib.child_watch_add(0, this._setup_pid,
                             Lang.bind(this, this._child_watch_cb), null);
    },

    _restart_item_shell_activate_cb: function(item, event, user_data) {
        this._is_restart = true;
        this._bus.exit(true);
    },

    _quit_item_shell_activate_cb: function(item, event, user_data) {
        this._bus.exit(false);
    },

    _set_im_icon: function(icon_name, label) {
        if (icon_name == null) {
            icon_name = ICON_ENGINE;
        }
        if (this._indicator != null) {
            if (icon_name[0] == '/') {
                let paths = null;
                let n_elements = 0;
                icon_name = GLib.path_get_basename(icon_name);
                if (icon_name.indexOf('.') >= 0) {
                    icon_name = icon_name.substr(0, icon_name.lastIndexOf('.'));
                }
            }
            if (label != null) {
                this._indicator.setLabel(label);
            } else {
                this._indicator.setIcon(icon_name);
            }
        } else {
            if (icon_name[0] == '/') {
                this._status_icon.set_from_file(icon_name);
            } else {
                this._status_icon.set_from_icon_name(icon_name);
            }
        }
    },

    _set_im_name: function(name) {
        this._language_bar.set_im_name(name);
    },

    set_cursor_location: function(panel, x, y, w, h) {
        this._candidate_panel.set_cursor_location(x, y, w, h);
    },

    update_preedit_text: function(panel, text, cursor_pos, visible) {
        this._candidate_panel.update_preedit_text(text, cursor_pos, visible);
    },

    show_preedit_text: function(panel) {
        this._candidate_panel.show_preedit_text();
    },

    hide_preedit_text: function(panel) {
        this._candidate_panel.hide_preedit_text();
    },

    update_auxiliary_text: function(panel, text, visible) {
        this._candidate_panel.update_auxiliary_text(text, visible);
    },

    show_auxiliary_text: function(panel) {
        this._candidate_panel.show_auxiliary_text();
    },

    hide_auxiliary_text: function(panel) {
        this._candidate_panel.hide_auxiliary_text();
    },

    update_lookup_table: function(panel, lookup_table, visible) {
        this._candidate_panel.update_lookup_table(lookup_table, visible);
    },

    show_lookup_table: function(panel) {
        this._candidate_panel.show_lookup_table();
    },

    hide_lookup_table: function(panel) {
        this._candidate_panel.hide_lookup_table();
    },

    page_up_lookup_table: function(panel) {
        this._candidate_panel.page_up_lookup_table();
    },

    page_down_lookup_table: function(panel) {
        this._candidate_panel.page_down_lookup_table();
    },

    cursor_up_lookup_table: function(panel) {
        this._candidate_panel.cursor_up_lookup_table();
    },

    cursor_down_lookup_table: function(panel) {
        this._candidate_panel.cursor_down_lookup_table();
    },

    show_candidate_window: function(panel) {
        this._candidate_panel.show_all();
    },

    hide_candidate_window: function(panel) {
        this._candidate_panel.hide_all();
    },

    register_properties: function(panel, props) {
        this._language_bar.register_properties(props);
    },

    update_property: function(panel, props) {
        this._language_bar.update_property(props);
    },

    focus_in: function(panel, path) {
        this.reset();
        this._focus_ic = IBus.InputContext.get_input_context(path,
                                                             this._bus.get_connection());
        let enabled = this._focus_ic.is_enabled();
        this._language_bar.set_enabled(enabled);
        if (this._active_engine != null) {
            this._focus_ic.set_engine(this._active_engine.name);
            this._active_engine = null;
        }

        if (enabled == false) {
            this._set_im_icon(ICON_KEYBOARD, null);
            this._set_im_name(null);
        } else {
            let engine = this._focus_ic.get_engine();
            if (engine) {
                this._set_im_icon(engine.icon, engine.language.substring(0,2));
                this._set_im_name(engine.longname);
            } else {
                this._set_im_icon(ICON_KEYBOARD, null);
                this._set_im_name(null);
            }
        }
    },

    focus_out: function(panel, path) {
        this.reset();
        this._focus_ic = null;
        this._language_bar.set_enabled(false);
        this._set_im_icon(ICON_KEYBOARD, null);
        this._set_im_name(null);
    },

    state_changed: function(panel) {
        if (this._focus_ic == null) {
            return;
        }

        let enabled = this._focus_ic.is_enabled();
        this._language_bar.set_enabled(enabled);

        if (enabled == false) {
            this.reset();
            this._set_im_icon(ICON_KEYBOARD, null);
            this._set_im_name(null);
        } else {
            let engine = this._focus_ic.get_engine();
            if (engine) {
                this._set_im_icon(engine.icon, engine.language.substring(0,2));
                this._set_im_name(engine.longname);
            } else {
                this._set_im_icon(ICON_KEYBOARD, null);
                this._set_im_name(null);
            }
        }
    },

    is_restart: function() {
        return this._is_restart;
    },

    restart: function(bus) {
        this._init_bus(bus);
        this._is_restart = false;
    },

    reset: function(ic) {
        //this._candidate_panel.reset();
        //this._language_bar.reset();
    },
};
